/*
 * Copyright (C) 2013 Avalpa Digital Engineering srl, All rights reserved.
 */

#define MULTICAST

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <time.h>

#include "api.h"

#define TS_PACKET_SIZE 188

long long int usecDiff(struct timespec* time_stop, struct timespec* time_start)
{
	long long int temp = 0;
	long long int utemp = 0;
		   
	if (time_stop && time_start) {
		if (time_stop->tv_nsec >= time_start->tv_nsec) {
			utemp = time_stop->tv_nsec - time_start->tv_nsec;    
			temp = time_stop->tv_sec - time_start->tv_sec;
		} else {
			utemp = time_stop->tv_nsec + 1000000000 - time_start->tv_nsec;       
			temp = time_stop->tv_sec - 1 - time_start->tv_sec;
		}		if (temp >= 0 && utemp >= 0) {
			temp = (temp * 1000000000) + utemp;
        	} else {
			fprintf(stderr, "start time %ld.%ld is after stop time %ld.%ld\n", time_start->tv_sec, time_start->tv_nsec, time_stop->tv_sec, time_stop->tv_nsec);
			temp = -1;
		}
	} else {
		fprintf(stderr, "memory is garbaged?\n");
		temp = -1;
	}
        return temp / 1000;
}

int main (int argc, char *argv[]) {

    char* tsfile;
    unsigned char write_buf[TS_PACKET_SIZE];

    //TPS tps;
    Byte DevNo = 0;
    int MaxGain = 0;
    int MinGain = 0;

    // for supporting IT950x or IT951x
    HandleType handleType;

    Byte Tx_NumOfDev = 0;
    Word ChipType = 0;
    //TxModDriverInfo driverInfo;
    DEVICE_INFO driverInfo;
    int adjustoutputgain;
    int outgainvalue;
    Dword dwError = ERR_NO_ERROR;
    MODULATION_PARAM ChannelModulation_Setting;
    Dword Freq;
    Word Bw;
    Dword ChannelCapacity = 0;

    int transport_fd;
    int written;
    int completed;
    int retries;
    int len;
    unsigned long long int packet_time;
    unsigned long long int real_time;    
    struct timespec time_start;
    struct timespec time_stop;
    struct timespec nano_sleep_packet;

    memset(&time_start, 0, sizeof(time_start));
    memset(&time_stop, 0, sizeof(time_stop));
    memset(&nano_sleep_packet, 0, sizeof(nano_sleep_packet));

    if(argc < 11 ) {
	fprintf(stderr, "Usage: %s file.ts device_id frequency bandwitdh fec gi kmode qam cell_id output_gain\n", argv[0]);
	fprintf(stderr, "Example: %s test.ts 1 810000 8000 16 2/3 1/4 8 0 0\n", argv[0]);
	fprintf(stderr, "means: send file test.ts on device 1 at 810MHz frequency, 8MHz bandwidth, 16qam, 2/3 fec, 1/4 gi, 8k mode, 0 cell id, 0 gain\n");
	fprintf(stderr, "possible values are:\n");
	fprintf(stderr, "	N device_number\n");
	fprintf(stderr, "	KHz Frequency\n");
	fprintf(stderr, "	2000-8000 KHz bandwidth\n");
	fprintf(stderr, "	4|16|32 qpsk, 16qam, 32qam\n");
	fprintf(stderr, 	"1/2|2/3|3/4|5/6|7/8 fec\n");
	fprintf(stderr, 	"1/16|1/32|1/4|1/8 guard interval\n");
	fprintf(stderr, "	2|8 2k or 8k mode\n");
	fprintf(stderr, "	id tps-cell_id\n");
	fprintf(stderr, "	gain output gain\n");
	fprintf(stderr, "NOTE for IT951X: if you put a device_id as a 'negative' number, the app will try to open device\n");
	fprintf(stderr, "                 /dev/usb-it951x and not /dev/usb-it950x.. i.e. a new Eagle II chip.\n");
	fprintf(stderr, "                 BEWARE: '-0' means open open /dev/usb-it951x0, albeit a negative 0 is meaningless.\n");
	return 0;
    } 

    tsfile = argv[1];
    transport_fd = open(tsfile, O_RDONLY);
    if(transport_fd < 0) {
	fprintf(stderr, "can't open file %s\n", tsfile);
	return 0;
    } 
 
   // quick hack, if i put a -X in the dev number, it means this is a EagleII IT951X device for num X
    if (argv[2][0] == '-') {
      // EagleII == IT951X
      handleType = EAGLEII;
      argv[2][0] = '0'; // remove the minus
    } else {
      // Eagle == IT950X
      handleType = EAGLEI;
    } 
    DevNo = atoi(argv[2]);

        /* Open handle Power On and Check device information */
        if((dwError = g_ITEAPI_TxDeviceInit(handleType, DevNo)) == ERR_NO_ERROR)
                printf("g_ITEAPI_TxDeviceInit ok\n");
        else
                printf("g_ITEAPI_TxDeviceInit fail\n");

        if((dwError = g_ITEAPI_TxGetNumOfDevice(&Tx_NumOfDev, DevNo)) == ERR_NO_ERROR)
                printf("%d Devices\n", Tx_NumOfDev);
        else
                printf("g_ITEAPI_TxGetNumOfDevice error\n");

    
    if((dwError = g_ITEAPI_TxGetChipType(&ChipType, DevNo)) != ERR_NO_ERROR) {
	fprintf(stderr, "g_ITE_TxGetChipType fail\n");
	return 0;
    }

    if ((dwError = g_ITEAPI_TxGetDrvInfo(&driverInfo, DevNo))) {
	fprintf(stderr, "Get Driver Info failed 0x%lu!\n", dwError);
	return 0;
    } else {
	fprintf(stderr, "g_ITEAPI_GetDrvInfo ok\n");		
	fprintf(stderr, "DriverInfo.DriverVerion  = %s\n", driverInfo.DriverVerion);
	fprintf(stderr, "DriverInfo.APIVerion     = %s\n", driverInfo.APIVerion);
	fprintf(stderr, "DriverInfo.FWVerionLink  = %s\n", driverInfo.FWVerionLink);
	fprintf(stderr, "DriverInfo.FWVerionOFDM  = %s\n", driverInfo.FWVerionOFDM);
	fprintf(stderr, "DriverInfo.Company       = %s\n", driverInfo.Company);
	fprintf(stderr, "DriverInfo.SupportHWInfo = %s\n", driverInfo.SupportHWInfo);
	fprintf(stderr, "DriverInfo.ProductID     = 0x%x\n", ChipType);
    }

    Freq = strtoul(argv[3], 0, 0);
    Bw = atoi(argv[4]);

    if ((dwError = g_ITEAPI_TxSetChannel(Freq, Bw, DevNo)) != ERR_NO_ERROR) {
	fprintf(stderr, "g_ITEAPI_TxSetChannel error\n");
	return 0;
    }

    switch(atoi(argv[5])) {
	case 4: ChannelModulation_Setting.constellation = 0; break;
	case 16: ChannelModulation_Setting.constellation = 1; break;
	case 64: ChannelModulation_Setting.constellation = 2; break;
	default: 
	    fprintf(stderr, "unsupported constallation\n"); return 0;
    }

    argv[6][1] = 0; /* force 1 digit only */
    switch(atoi(argv[6])) {
	case 1: ChannelModulation_Setting.highCodeRate = 0; break;
	case 2: ChannelModulation_Setting.highCodeRate = 1; break;
	case 3: ChannelModulation_Setting.highCodeRate = 2; break;
	case 5: ChannelModulation_Setting.highCodeRate = 3; break;
	case 7: ChannelModulation_Setting.highCodeRate = 4; break;
	default: 
	    fprintf(stderr, "unsupported fec\n"); return 0;
    }


    switch(atoi(argv[7]+2)) { /* jump "1/" */
	case 32: ChannelModulation_Setting.interval = 0; break;
	case 16: ChannelModulation_Setting.interval = 1; break;
	case 8: ChannelModulation_Setting.interval = 2; break;
	case 4: ChannelModulation_Setting.interval = 3; break;
	default: 
	    fprintf(stderr, "unsupported guard interval\n"); return 0;
    }

    if (Bw == 2000) {
	fprintf(stderr, "Only 2k supported for 2MHz\n");
	ChannelModulation_Setting.transmissionMode = 0;
    } else {
        switch(atoi(argv[8])) {
	    case 2: ChannelModulation_Setting.transmissionMode = 0; break;
	    case 8: ChannelModulation_Setting.transmissionMode = 1; break;
	    default: 
		fprintf(stderr, "unsupported transmission mode\n"); return 0;
        }
    }
	
    fprintf(stderr, "Frequency = %lu KHz\n", Freq);
    fprintf(stderr, "Bandwidth = %d MHz\n", Bw);
    
    switch(ChannelModulation_Setting.constellation) {
	    case 0: fprintf(stderr,"Constellation: QPSK\n"); break;
	    case 1: fprintf(stderr,"Constellation: 16QAM\n"); break;
	    case 2: fprintf(stderr,"Constellation: 64QAM\n"); break;
	    default: break;
    }

    switch(ChannelModulation_Setting.highCodeRate) {
	    case 0: fprintf(stderr,"Code Rate: 1/2\n"); break;
	    case 1: fprintf(stderr,"Code Rate: 2/3\n"); break;
	    case 2: fprintf(stderr,"Code Rate: 3/4\n"); break;
	    case 3: fprintf(stderr,"Code Rate: 5/6\n"); break;
	    case 4: fprintf(stderr,"Code Rate: 7/8\n"); break;
	    default: break;
    }
	
    switch(ChannelModulation_Setting.interval) {
	    case 0: fprintf(stderr,"Interval: 1/32\n"); break;
	    case 1: fprintf(stderr,"Interval: 1/16\n"); break;
	    case 2: fprintf(stderr,"Interval: 1/8\n"); break;
	    case 3: fprintf(stderr,"Interval: 1/4\n"); break;
	    default: break;
    }
    
    switch(ChannelModulation_Setting.transmissionMode) {
	    case 0: printf("Transmission Mode: 2K\n"); break;
	    case 1: printf("Transmission Mode: 8K\n"); break;
	    default: break;
    }
    
	
    if ((dwError = g_ITEAPI_TxSetChannelModulation(ChannelModulation_Setting, DevNo)) != ERR_NO_ERROR)
	    fprintf(stderr, "g_ITEAPI_TxSetChannelModulation error!!, %lu ******\n", dwError);
	
	
	

    ChannelCapacity = Bw * 1000;
    ChannelCapacity = ChannelCapacity * ( ChannelModulation_Setting.constellation * 2 + 2);

    switch (ChannelModulation_Setting.interval) {
	case 0: //1/32
		ChannelCapacity = ChannelCapacity*32/33;
		break;
	case 1: //1/16
		ChannelCapacity = ChannelCapacity*16/17;
		break;
	case 2: //1/8
		ChannelCapacity = ChannelCapacity*8/9;
		break;
	case 3: //1/4
		ChannelCapacity = ChannelCapacity*4/5;
		break;
    }
    switch (ChannelModulation_Setting.highCodeRate) {
	case 0: //1/2
		ChannelCapacity = ChannelCapacity*1/2;
		break;
	case 1: //2/3
		ChannelCapacity = ChannelCapacity*2/3;
		break;
	case 2: //3/4
		ChannelCapacity = ChannelCapacity*3/4;
		break;
	case 3: //5/6
		ChannelCapacity = ChannelCapacity*5/6;
		break;
	case 4: //7/8
		ChannelCapacity = ChannelCapacity*7/8;
		break;
    }

    ChannelCapacity = ChannelCapacity/544*423;
    fprintf(stderr, "Channel Capacity is %lu bps\n",ChannelCapacity);

    g_ITEAPI_TxGetGainRange(Freq, Bw, &MaxGain, &MinGain, DevNo);
    fprintf(stderr, "MinGain: %d, MaxGain: %d\n", MinGain, MaxGain);

    adjustoutputgain = atoi(argv[10]);
    if(g_ITEAPI_TxAdjustOutputGain(adjustoutputgain, &outgainvalue, DevNo) != ERR_NO_ERROR) {
	    fprintf(stderr, "g_ITEAPI_TxAdjustOutputGain error\n");
	    return 0;
    }

/*
    memset(&tps, 0, sizeof(tps));
    tps.cellid = atoi(argv[9]);
    if(g_ITEAPI_TxSetTPS(tps, DevNo) != ERR_NO_ERROR) {
	    fprintf(stderr, "g_ITEAPI_TxSetTPS error\n");
	    return 0;
    }
 */

    completed = 0;
    packet_time = 0;
    real_time = 0;
    nano_sleep_packet.tv_nsec = 665778; /* 1 ts packet at 100 mbps*/
    g_ITEAPI_StartTransfer(DevNo);
    clock_gettime(CLOCK_MONOTONIC, &time_start);
    while (!completed) {

	    clock_gettime(CLOCK_MONOTONIC, &time_stop);
	    real_time = usecDiff(&time_stop, &time_start);
	    while ((real_time * ChannelCapacity > packet_time * 1000000)&!completed) { /* theorical bits against written bits */
		len = read(transport_fd, write_buf, TS_PACKET_SIZE);
		if(len < 0) {
		    fprintf(stderr, "ts file read error \n");
		    completed = 1;
		} else if (len == 0) {
		    fprintf(stderr, "ts sent done\n");
		    completed = 1;
		} else {
                    retries=0;
retry:
		    written = g_ITEAPI_TxSendTSData(write_buf, len, DevNo);
		    if(written != 0) {
                        if (retries <10) {
                          usleep(100);
                          printf("packet_time %lld, len %d, written %d, rewrite %d\n", packet_time, len, written, retries);
                          retries++;
                          goto retry;
                        } else {
			  perror("send(): error ");
			  completed = 1;
                        }
		    } else {
			packet_time += TS_PACKET_SIZE * 8;
		    }
		}
	    } 
	    nanosleep(&nano_sleep_packet, 0);
    }
    g_ITEAPI_StopTransfer(DevNo);
    g_ITEAPI_TxDeviceExit(DevNo);
    close(transport_fd);
    return 0;
}
